==> pathena-0.3.3.tgz: .pathena/server/client_ops.py <==

# Initialize the PostgreSQL-resident server functions for the Pathena
# GUI client.  This module is not needed (and should not be imported)
# by the indexing process or other components.

import sys, os, time

#=================================


import string, glob, shutil
from operator import add, and_
from StringIO import StringIO
from traceback import print_exc

from file_type_attribs import *

_temp_file = new_temp_file_name()
_temp_file_decomp = new_temp_file_name()



# Retrieve a file's text content either by direct conversion or by
# check if a file's content has been cached in the database, then by
# conversion.  If any problems, return empty string.
### consider using file_id for lookup instead of path

def get_cached_content(type_class, path, usage=0):
    if not type_class.cache_conversion():
        return decompress_convert(type_class, path, usage)
    db_attribs = _GD["execute_query"](
                     "select text_content from file_descriptors "
                     "where abs_path = '%s'" % quote_quote(path))
    if db_attribs: content = db_attribs[0]["text_content"]
    else:          return decompress_convert(type_class, path, usage)
    if not content: return decompress_convert(type_class, path, usage)
    return type_class.convert_cached_content(content, usage)

def decompress_convert(type_class, path, usage=0):
    base, ext = os.path.splitext(path)
    if ext[1:] in compress_ext_command and \
       os.path.splitext(base)[1][1:] in file_ext_class:
        decompress_cmd = compress_ext_command[ext[1:]]
        make_shell_command('%s > %s' % (decompress_cmd, _temp_file))(path)
        text = type_class.convert_file(_temp_file, usage)
        try:    os.remove(_temp_file)
        except: pass
        return text
    else:
        return type_class.convert_file(path, usage)


# Ask Tsearch2 to compute a headline for the designated query by
# extracting text from the file.  Truncate output to LIMIT words.
# Arg 'terms' is a string of terms using newlines as delimiters.
# If non-alphanumeric terms exist (called identifiers below), use
# instead a regexp search of the file text.  If mode==1, the request is
# for missed terms; use more liberal searching.

def get_headline(type_class, path_name, terms, limit, specials='', mode=0):
    terms = terms.split('\n')
    contents = get_cached_content(type_class, path_name, 2)   # headline mode
    if mode == 1:
        headline = make_identifier_headline(contents, terms,
                                            min(5, limit // len(terms)),
                                            specials, 1)
        return ' '.join(headline.split())

    terms_alnums = zip(terms, [ term.isalnum() for term in terms ])
    alnum_terms  = [ t for t, alnum in terms_alnums if alnum ]
    ident_terms = terms   # alnum_terms will be stripped out below
    headline = ''
    if alnum_terms and type_class.file_indexing_attrib != 'path_list':
        ident_terms  = [ t for t, alnum in terms_alnums if not alnum ]
        if ident_terms: w_limits = (min( 5, limit-6), limit - 5)
        else:           w_limits = (min(10, limit-1), limit)
        headline = make_tsearch_headline(contents, path_name,
                                         Tsearch_terms(alnum_terms), *w_limits)
    if ident_terms:
        i_limit = min(5, limit // len(ident_terms))
        i_head = make_identifier_headline(contents, ident_terms,
                                          i_limit, specials, 0)
        if headline: headline = headline + '  ...  ' + i_head
        else:        headline = i_head
    return ' '.join(headline.split())


# Fake headline generation for search terms having identifier form.
# Return next N words after term.

def make_identifier_headline(contents, terms, limit, specials, mode):
    headlines = [ general_headline_term(contents, term, limit, specials, mode)
                  for term in terms if term[0] not in '-!' ]
    return '  ...  '.join(filter(None, headlines))

### will not put ... items in proper order

def general_headline_term(contents, term, limit, specials, mode):
    # All of the following patterns ignore case. The term should
    # appear as the second group matched.
    if mode == 1:
        # follow-up for previously missed terms; match term in any context
        term_pattern = re.compile(r'(?i)((?=.))(%s)' % term)
#    elif term.isalnum():
#        term_pattern = re.compile(r'(?i)(\b)(%s)\b' % term)
    elif specials:
        # when special chars exist, context must exclude them
        # certain specials need to be protected before substitution in patterns
        esc_spec = specials.replace('-','\-').replace(']','\]')
        term_pattern = re.compile(r'(?i)([^%s\w]|^)(%s)(?=[^%s\w]|$)' %
                                  (esc_spec, re.escape(term), esc_spec))
    else:
        # all alphanums, file names; context is word boundaries
        term_pattern = re.compile(r'(?i)(\b)(%s)\b' % term)
    term_match = term_pattern.search(contents)
    if not term_match: return ''
    start = term_match.end()
    word_pattern = re.compile('\S+')
    max_chars = limit * 6   # limit total chars as well as num words
    words, num_words, cum_chars = [], 0, 0
    while num_words < limit:
        word_match = word_pattern.search(contents, start)
        if not word_match: break
        word = word_match.group()
        words.append(word)
        cum_chars += len(word)
        if cum_chars > max_chars: break
        start = word_match.end()
        num_words += 1
    return '<b>' + term_match.group(2) + '</b> ' + ' '.join(words)


# Let Tsearch2 make the headline.  Be sure min_words < max_words.

def make_tsearch_headline(contents, path, query, min_words=10, max_words=25):
#    start = time.time()
    # replace NUL chars and single quotes
    contents = contents.replace("\x00", " ").replace("'", " ")  # "''")
#    check_for_unicode(path, text)
    query_fn = "to_tsquery('file_content', '%s')" % query
    min_max_words = 'MinWords=%s, MaxWords=%s' % (min_words, max_words)
    results = \
        _GD["execute_query"]("select headline('file_content', '%s', %s, '%s')"
                             % (contents, query_fn, min_max_words))

    if results:
        return results[0]["headline"]
    else:
        return ""     ### add failure indication???


#=================================


def init_support_ops(GD):
    global _GD, PATHENA
    _GD = GD
    PATHENA = GD["PATHENA"]

    ### add exception recording and announcement
    try:
        for handler in glob.glob(os.path.join(PATHENA, 'types', '*.py')):
            __import__(os.path.splitext(os.path.basename(handler))[0])
    except:
        GD["emit_plpy_log_entry"](
            "Failed to import types module '%s'." % handler)

    GD["get_cached_content"] = get_cached_content
    GD["get_headline"]       = get_headline

    GD["emit_plpy_log_entry"]('Client startup complete.')

==> pathena-0.3.4.tgz: .pathena/server/client_ops.py <==

# Initialize the PostgreSQL-resident server functions for the Pathena
# GUI client.  This module is not needed (and should not be imported)
# by the indexing process or other components.

import sys, os, time

#=================================


import string, glob, shutil
from operator import add, and_
from StringIO import StringIO
from traceback import print_exc

from file_type_attribs import *

_temp_file = new_temp_file_name()
_temp_file_decomp = new_temp_file_name()



# Retrieve a file's text content either by direct conversion or by
# check if a file's content has been cached in the database, then by
# conversion.  If any problems, return empty string.
### consider using file_id for lookup instead of path

def get_cached_content(type_class, path, usage=0):
    if not type_class.cache_conversion():
        return decompress_convert(type_class, path, usage)
    db_attribs = _GD["execute_query"](
                     "select text_content from file_descriptors "
                     "where abs_path = '%s'" % quote_quote(path))
    if db_attribs: content = db_attribs[0]["text_content"]
    else:          return decompress_convert(type_class, path, usage)
    if not content: return decompress_convert(type_class, path, usage)
    return type_class.convert_cached_content(content, usage)

def decompress_convert(type_class, path, usage=0):
    base, ext = os.path.splitext(path)
    ext = ext[1:].lower()
    if ext in compress_ext_command and \
       os.path.splitext(base)[1][1:].lower() in file_ext_class:
        decompress_cmd = compress_ext_command[ext]
        make_shell_command('%s > %s' % (decompress_cmd, _temp_file))(path)
        text = type_class.convert_file(_temp_file, usage)
        try:    os.remove(_temp_file)
        except: pass
        return text
    else:
        return type_class.convert_file(path, usage)


# Ask Tsearch2 to compute a headline for the designated query by
# extracting text from the file.  Truncate output to LIMIT words.
# Arg 'terms' is a string of terms using newlines as delimiters.
# If non-alphanumeric terms exist (called identifiers below), use
# instead a regexp search of the file text.  If mode==1, the request is
# for missed terms; use more liberal searching.

def get_headline(type_class, path_name, terms, limit, specials='', mode=0):
    terms = terms.split('\n')
    contents = get_cached_content(type_class, path_name, 2)   # headline mode
    if mode == 1:
        headline = make_identifier_headline(contents, terms,
                                            min(5, limit // len(terms)),
                                            specials, 1)
        return ' '.join(headline.split())

    terms_alnums = zip(terms, [ term.isalnum() for term in terms ])
    alnum_terms  = [ t for t, alnum in terms_alnums if alnum ]
    ident_terms = terms   # alnum_terms will be stripped out below
    headline = ''
    if alnum_terms and type_class.file_indexing_attrib != 'path_list':
        ident_terms  = [ t for t, alnum in terms_alnums if not alnum ]
        if ident_terms: w_limits = (min( 5, limit-6), limit - 5)
        else:           w_limits = (min(10, limit-1), limit)
        headline = make_tsearch_headline(contents, path_name,
                                         Tsearch_terms(alnum_terms), *w_limits)
    if ident_terms:
        i_limit = min(5, limit // len(ident_terms))
        i_head = make_identifier_headline(contents, ident_terms,
                                          i_limit, specials, 0)
        if headline: headline = headline + '  ...  ' + i_head
        else:        headline = i_head
    return ' '.join(headline.split())


# Fake headline generation for search terms having identifier form.
# Return next N words after term.

def make_identifier_headline(contents, terms, limit, specials, mode):
    headlines = [ general_headline_term(contents, term, limit, specials, mode)
                  for term in terms if term[0] not in '-!' ]
    return '  ...  '.join(filter(None, headlines))

### will not put ... items in proper order

def general_headline_term(contents, term, limit, specials, mode):
    # All of the following patterns ignore case. The term should
    # appear as the second group matched.
    if mode == 1:
        # follow-up for previously missed terms; match term in any context
        term_pattern = re.compile(r'(?i)((?=.))(%s)' % term)
#    elif term.isalnum():
#        term_pattern = re.compile(r'(?i)(\b)(%s)\b' % term)
    elif specials:
        # when special chars exist, context must exclude them
        # certain specials need to be protected before substitution in patterns
        esc_spec = specials.replace('-','\-').replace(']','\]')
        term_pattern = re.compile(r'(?i)([^%s\w]|^)(%s)(?=[^%s\w]|$)' %
                                  (esc_spec, re.escape(term), esc_spec))
    else:
        # all alphanums, file names; context is word boundaries
        term_pattern = re.compile(r'(?i)(\b)(%s)\b' % term)
    term_match = term_pattern.search(contents)
    if not term_match: return ''
    start = term_match.end()
    word_pattern = re.compile('\S+')
    max_chars = limit * 6   # limit total chars as well as num words
    words, num_words, cum_chars = [], 0, 0
    while num_words < limit:
        word_match = word_pattern.search(contents, start)
        if not word_match: break
        word = word_match.group()
        words.append(word)
        cum_chars += len(word)
        if cum_chars > max_chars: break
        start = word_match.end()
        num_words += 1
    return '<b>' + term_match.group(2) + '</b> ' + ' '.join(words)


# Let Tsearch2 make the headline.  Be sure min_words < max_words.

def make_tsearch_headline(contents, path, query, min_words=10, max_words=25):
#    start = time.time()
    # replace NUL chars and single quotes
    contents = contents.replace("\x00", " ").replace("'", " ")  # "''")
#    check_for_unicode(path, text)
    query_fn = "to_tsquery('file_content', '%s')" % query
    min_max_words = 'MinWords=%s, MaxWords=%s' % (min_words, max_words)
    results = \
        _GD["execute_query"]("select headline('file_content', '%s', %s, '%s')"
                             % (contents, query_fn, min_max_words))

    if results:
        return results[0]["headline"]
    else:
        return ""     ### add failure indication???


#=================================


def init_support_ops(GD):
    global _GD, PATHENA
    _GD = GD
    PATHENA = GD["PATHENA"]

    ### add exception recording and announcement
    try:
        for handler in glob.glob(os.path.join(PATHENA, 'types', '*.py')):
            __import__(os.path.splitext(os.path.basename(handler))[0])
    except:
        GD["emit_plpy_log_entry"](
            "Failed to import types module '%s'." % handler)

    GD["get_cached_content"] = get_cached_content
    GD["get_headline"]       = get_headline

    GD["emit_plpy_log_entry"]('Client startup complete.')
